/*
 * kernInstIoctls.h - declarations for ioctl's implemented by kerninst driver
 */

#ifndef _KERNINST_LINUX_IOCTL_
#define _KERNINST_LINUX_IOCTL_

#include <linux/ioctl.h>
#include "hwcounter_p4.h"

typedef unsigned int kptr_t;

struct kernInstTrapMapping {
   kptr_t trapAddr;
   kptr_t patchAddr;
};

struct kernInstWrite1Insn {
   kptr_t destAddrInKernel;
   const char *bytes;
   unsigned length;
};

struct kernInstWrite1UndoableInsn {
   kptr_t destAddrInKernel;
   const char *newval;
   unsigned length;
   unsigned howSoon;
      /* 0 --> undo first  1 --> undo after 0's */

   /* initially undefined, filled in by /dev/kerninst: */
   char *origval;
};

struct kernInstChangeUndoHowSoon {
   kptr_t addrInKernel;
   unsigned newHowSoon;
      /* 0 --> undo first  1 --> undo after 0's */
};

struct kernInstUndo1Write {
   kptr_t addr;
   const char *oldval_to_set_to; /* just for asserting */
   unsigned length;
};

#define ALLOC_TYPE_DATA 0
#define ALLOC_TYPE_TRY_NUCLEUS 1
#define ALLOC_TYPE_BELOW_NUCLEUS 2
#define ALLOC_TYPE_NUCLEUS 3
#define ALLOC_TYPE_KMEM32 4

struct kernInstAllocStruct {
   unsigned nbytes;
   unsigned requiredAlignment;
      /* e.g. 8192 if you want to be able to mmap the result */
      /* or e.g. 32 if you want the result to be icache-block aligned */
   unsigned allocType; /* See ALLOC_ constants above */

   /* The remaining params are filled in for you: */
   kptr_t result;
   kptr_t result_mappable;
      /* filled in for you; differs from 'result' only if we allocated from the
         nucleus text and need to present a more mappable kernel vaddr */
};

struct kernInstFreeStruct {
   kptr_t kernelAddr;
   kptr_t mappedKernelAddr;
   unsigned nbytes;

   /* the driver keeps around enough housekeeping info to translate these values
      into actual addresses & nbytes (could differ if padding was added behind the
      scenes to fulfill alignment requirements */
};

struct kernInstSymtabGetStruct {
   kptr_t buffer; /* buffer is in user space...had better be big enough! */
   unsigned buffer_size;

   unsigned actual_amt_copied;
};

struct kernInstOffsetInfo {
   unsigned t_proc_offset; /* offset of processor field in task_struct */
   unsigned t_pid_offset; /* offset of pid field in task_struct */
};

struct kernInstCallSiteInfo {
   /* Address of the call site */
   kptr_t addrFrom;

   /* Number of callees for this site. It is set by the
      KERNINST_KNOWN_CALLEES_SIZE ioctl and checked by the
      KERNINST_KNOWN_CALLEES_GET ioctl */
   unsigned num_callees;

   /* Buffer in user space...had better be big enough! We fill it in the
      KERNINST_KNOWN_CALLEES_GET ioctl */
   kptr_t buffer; 
};

struct kernInstCSwitchPoints {
   unsigned type; /* 0 == cswitch_out, 1 == cswitch_in */
   kptr_t *addrs;
   unsigned addrs_len;
}; 

#define KERNINST_MAGIC 0xCC /* opcode for int3 */

/* these are the ioctls */
#define KERNINST_GET_OFFSET_INFO _IOR(KERNINST_MAGIC, 1, struct kernInstOffsetInfo)

#define KERNINST_REGISTER_TRAP_PATCH_MAPPING _IOW(KERNINST_MAGIC, 3, struct kernInstTrapMapping)
#define KERNINST_UPDATE_TRAP_PATCH_MAPPING _IOW(KERNINST_MAGIC, 4, struct kernInstTrapMapping)
#define KERNINST_UNREGISTER_TRAP_PATCH_MAPPING _IOW(KERNINST_MAGIC, 5, kptr_t)

#define KERNINST_ALLOC_BLOCK _IOWR(KERNINST_MAGIC, 6, struct kernInstAllocStruct)
#define KERNINST_FREE_BLOCK _IOW(KERNINST_MAGIC, 7, struct kernInstFreeStruct)
#define KERNINST_ALLOC_CPU_64BIT_PSEUDOREGS _IOR(KERNINST_MAGIC, 8, kptr_t)

#define KERNINST_SYMTAB_STUFF_SIZE _IOR(KERNINST_MAGIC, 9, unsigned)
#define KERNINST_SYMTAB_STUFF_GET _IOWR(KERNINST_MAGIC, 10, struct kernInstSymtabGetStruct)

#define KERNINST_WRITE1INSN _IOW(KERNINST_MAGIC, 11, struct kernInstWrite1Insn)
#define KERNINST_WRUNDOABLE_REG _IOWR(KERNINST_MAGIC, 12, struct kernInstWrite1UndoableInsn)
#define KERNINST_UNDO_WR_REG _IOW(KERNINST_MAGIC, 13, struct kernInstUndo1Write)
#define KERNINST_CHG_UNDO_HOWSOON _IOW(KERNINST_MAGIC, 14, struct kernInstChangeUndoHowSoon)

#define KERNINST_VIRTUAL_TO_PHYSICAL_ADDR _IOWR(KERNINST_MAGIC, 15, kptr_t)

#define KERNINST_KNOWN_CALLEES_SIZE _IOWR(KERNINST_MAGIC, 16, struct kernInstCallSiteInfo) 
#define KERNINST_KNOWN_CALLEES_GET _IOW(KERNINST_MAGIC, 17, struct kernInstCallSiteInfo)

#define KERNINST_GET_TICK_RAW _IOR(KERNINST_MAGIC, 18, unsigned)

#define KERNINST_CALLONCE _IO(KERNINST_MAGIC, 19)

#define KERNINST_GET_ALLVTIMERS_ADDR _IOR(KERNINST_MAGIC, 20, kptr_t)
#define KERNINST_ADD_TO_ALLVTIMERS _IOW(KERNINST_MAGIC, 21, kptr_t)
#define KERNINST_REMOVE_FROM_ALLVTIMERS _IOW(KERNINST_MAGIC, 22, kptr_t)

#define KERNINST_GET_BUG_UD2_SIZE _IOWR(KERNINST_MAGIC, 23, unsigned)

#define KERNINST_GET_PERF_CTR_STATE _IOR(KERNINST_MAGIC, 24, perfctr_state_t)
#define KERNINST_SET_PERF_CTR_STATE _IOW(KERNINST_MAGIC, 25, perfctr_state_t)
#define KERNINST_GET_PERF_CTR_VAL _IOWR(KERNINST_MAGIC, 26, perfctr_val_t)
#define KERNINST_SET_PERF_CTR_VAL _IOWR(KERNINST_MAGIC, 27, perfctr_val_t)

#define KERNINST_ENABLE_CSWITCH_HANDLING _IOW(KERNINST_MAGIC, 252, unsigned)
#define KERNINST_SET_JUST_TESTING_FLAG _IOW(KERNINST_MAGIC, 253, unsigned)
#define KERNINST_SET_DEBUG_OUTPUT _IOW(KERNINST_MAGIC, 254, unsigned)
#define KERNINST_RESET _IO(KERNINST_MAGIC, 255)


int kerninst_alloc_block(void *loggedAllocRequests,
                         struct kernInstAllocStruct *);
int kerninst_free_block(void *loggedAllocRequests,
                        struct kernInstFreeStruct *);

#endif

